/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 1
 * Digital I/O
 *********************************************************************************/
#define F_CPU 16000000
#include <avr\io.h>
#include <avr\interrupt.h>
#include <util/delay.h>
#include "math.h"

/************************************************************************ 
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRA			// Portul pe care conectam firele de date la LCD-ul
#define LcdDATA_PORT	PORTA
#define LcdDATA_PIN		PINA

#define LcdCMD_DDR		DDRA			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTA
#define LcdCMD_PIN		PINA

#define LcdD4			 PA7				// Pin-ul pentru firul de date D4 de pe LCD
#define LcdD5			 PA6				// Pin-ul pentru firul de date D5 de pe LCD
#define LcdD6			 PA5				// Pin-ul pentru firul de date D6 de pe LCD
#define LcdD7			 PA4				// Pin-ul pentru firul de date D7 de pe LCD

#define LcdRS			 PA3			    // Pinul pentru selectare operatie (LCD)
#define LcdRW			 PA2				// Pinul pentru Read/ Write (LCD)
#define LcdE			 PA1				// Pinul de Enable (LCD)

#define RW_LOW()				LcdCMD_PORT &= ~_BV(LcdRW)
#define RS_HIGH()				LcdCMD_PORT |= _BV(LcdRS)
#define RS_LOW()				LcdCMD_PORT &= ~_BV(LcdRS) 
#define ENABLE()				LcdCMD_PORT |= _BV(LcdE)
#define DISABLE()				LcdCMD_PORT &= ~_BV(LcdE)

#define LCD_INSTR_4wire 		0x28 	// 4 fire date, font 5x8
#define LCD_INSTR_display 		0x0C 	// Display On, Cursor On, Blinking On ( 1 Display Cursor Blink )
#define LCD_INSTR_clearDisplay  	0x01 	// Clear Display
#define LCD_INSTR_returnHome		0x02 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_nextLine 		0xC0 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_gotoCGRAM		0x40	// go to Character Generator RAM

#define WSIZE 1.5
/*************
 * API LCD   *
 *************/
void LCD_init();                // Initializare modul LCD.Trebuie apelata inainte de a se face orice operatie cu LCD-ul
void LCD_writeInstruction(unsigned char _instruction);	// Trimite o instructiune catre lcd (vezi datasheet)
void LCD_writeData(unsigned char _data);		// Trimite date catre LCD pentru afisare
void LCD_write(unsigned char _byte);			// trimite un bute catre LCD la modul general (nu conteaza daca e instructiune sau date)
void LCD_waitNotBusy();					// Asteptam pana cand lcd-ul devine disponibil pt o noua comanda
void LCD_waitInstructions(unsigned char _instructions);	// Asteapta un numar de cicli de ceas.
void LCD_print(char* _msg);				// Afiseaza imformatia pe LCD (doar 1 linie, primele 16 caractere)
void LCD_print2(char* _msg1, char* _msg2);		// Afisare pe 2 lini pe LCD
void LCD_printDecimal2u(unsigned int _n);		// Afisare numar in baza 10 pe LCD
void LCD_printHexa(unsigned int _n);			// Afisare numar in baza 16 pe LCD

int cycles = 0;
float time = 0.1;
int beat = 0;
int pc = 0;

int distfivecycles = 0;

void print_dist() {
	LCD_writeInstruction(1);
	LCD_print("Distanta Parcursa: ");
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_printDecimal2u(cycles*WSIZE);
	LCD_print(" m");
}

void print_speed_med() {
	LCD_writeInstruction(1);
	LCD_print("Viteza Medie: ");
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_printDecimal2u((int)cycles*WSIZE*3.6/time);
	LCD_print(" km/h");
}

void print_speed_now() {
	LCD_writeInstruction(1);
	LCD_print("Viteza Curenta: ");
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_printDecimal2u((int)distfivecycles*WSIZE*3.6);
	LCD_print(" km/h");
}

void print_data() {

	switch(pc) {
		case 0: print_dist(); break;
		case 1: print_speed_med(); break;
		case 2: print_speed_now(); break;
	} 
	
}

int main()
{
    /* TODO */
	int i=0;
	LCD_init();
	
	DDRB=0;
	PORTB=0xFF;
	while(1) {	
		
		if(!(PINB & (1<<PB0))) {
			cycles++;
			distfivecycles++;			
		}

		_delay_ms(100);

		if(distfivecycles>0) //PEDALAM
			time += 0.1;

		if(beat%15 == 0) 
			print_data();
	
		if(beat%45 == 0) {
			pc++;
			if(pc>2) pc=0;
		}

		if(beat%10 == 0) 			
			distfivecycles=0;
			
		
		

		beat++;
	}

	return 0;
}

/*****************************
 * IMPLEMENTARE API LCD.     *
 *****************************/

void LCD_init()
{
	LcdDATA_DDR |=  (1<<LcdD4)|(1<<LcdD5)|(1<<LcdD6)|(1<<LcdD7);	// setam pinii de date ca pini de iesire
	LcdCMD_DDR  |=  (1<<LcdRS)|(1<<LcdRW)|(1<<LcdE);				// setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();
	
	LcdCMD_PORT   &= ~(1<<LcdRS);									// Setam linia RS pe low
	LcdCMD_PORT   &= ~(1<<LcdRW);									// Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT  &= ~(1<<LcdD4)&~(1<<LcdD6)&~(1<<LcdD7); 		// Specificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT  |=  (1<<LcdD5);									// setam pinii de comenzi ca pini de iesire
	
	LcdCMD_PORT |=  (1<<LcdE);					// Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	LCD_waitInstructions(12);					// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE );				// Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); 		// Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); 	// Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06);					// Increment, no shift
	LCD_writeInstruction(0x01);					// Clear Display

	
}

void LCD_writeInstruction(unsigned char _instruction)
{			
	
	LCD_waitNotBusy(); 							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	RS_LOW();									// setam pinul RS pe low (low=instructiuni, high=date)
	RW_LOW();									// setam pinul RW pe low (suntem in modul de comenzi acum)
	LCD_write(_instruction);					// apelam procedura ce trimite byte-ul pe firele de date
}


void LCD_writeData(unsigned char _data)
{
	LCD_waitNotBusy(); 							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	RS_HIGH();									// setam pinul RS pe high (low=instructiuni, high=date)
	RW_LOW();									// setam pinul RW pe low (suntem in modul de date acum)
	LCD_write(_data);							// apelam procedura ce trimite byte-ul pe firele de date

}

void LCD_write(unsigned char _byte)
{
	unsigned char _byte2;
	
	_byte2 = _byte>>4;

	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte2, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
		
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte2, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte2, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte2, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	
	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
	
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
	LCD_waitInstructions(12);
}

void LCD_waitNotBusy()
{
_delay_ms(5);
/*
	unsigned char _loop = 1;

	while (_loop)
	{
		LcdDATA_DDR &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7);	// Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); 	// Dezactivam pullup resistor pentru pinii de in
		
		LcdCMD_PORT &= ~(1<<LcdE);					// Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1<<LcdRS);					// Setam pinul RS pe low
		LcdCMD_PORT |=  (1<<LcdRW);					// Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1<<LcdD7);				// Citim busy flag-ul
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
		
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

		LcdDATA_DDR |= (1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}*/
}


void LCD_waitInstructions(unsigned char _instructions)
{
	while (_instructions--)
		;
}


void LCD_printDecimal2u(unsigned int _n)
{
	unsigned char tmp=0,tmp2=0,tmp3=0;
	
	while(_n>=1000) {
		tmp3++;
		_n-=1000;
		
	}

	// Extragem sutele
	while(_n>=100) {
		tmp2++;
		_n-=100;
		
	}

	while(_n>=10){
		tmp++;
		_n-=10;
	}

	if(tmp3)
		LCD_writeData(tmp3+'0');

	if(tmp2)
		LCD_writeData(tmp2+'0');

	if(tmp)
		LCD_writeData(tmp+'0');

	LCD_writeData(_n+'0');
}

void LCD_printHexa(unsigned int _n)
{
	unsigned char _tmp = _n>>4;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
	_tmp = _n & 0x0F;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
}


void LCD_print(char* _msg)
{
	unsigned char i=0;
	for( ; _msg[i]!=0 && i<16; i++)
		LCD_writeData( _msg[i] );
}

void LCD_print2(char* _msg1, char* _msg2)
{
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}
